#!/usr/bin/env python
## Copyright (C) 2002   Martin Willemoes Hansen

## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

## Author:   Martin Willemoes Hansen
## Version:  0.0.2

import pygame
import sys
from pygame.locals import *
from common import *

USAGE = '''Usage: pykanoid_editor file_to_save [file_to_load]

In the editor you can use these key strokes:
   c   Clears the screen
   s   Saves the level
   ESC Quits

   bug reports goes to <mwh@sysrq.dk>'''

class Block (pygame.sprite.Sprite):
    def __init__ (self, x, y, number, id):
        pygame.sprite.Sprite.__init__ (self)

        self.rect = pygame.Rect (0, 0, 40, 20)
        self.rect.topleft = (x, y)
        self.number = number
        self.id = id

    def draw (self):
        self.image = gfx[self.number]
        screen.display.blit (self.image, self.rect)

    def clear (self):
        screen.display.blit (gfx ['bg1'], self.rect, self.rect)

    def update (self):
        self.number = (self.number + 1) % 8
        if (self.number != 0):
            self.draw()
        else:
            self.clear()

    def reset (self):
        self.number = -1
        self.update()

def main():
    argsLength = len (sys.argv)
    if argsLength > 3 or argsLength < 2:
        print USAGE
        sys.exit(1)
    global savefile
    savefile = sys.argv[1]

    pygame.init()
    global screen
    screen = Screen (480, 240, '-=* Pykanoid Level Editor *=-')
    load_files()

    #Load a map or create an empty
    global blocks
    blocks = pygame.sprite.Group()
    if (argsLength is 2):
        empty_level()
    else:
        load_level (sys.argv[2])

    game_loop()

def save_level():
    # sort
    blocksList = blocks.sprites()
    length = len (blocksList)

    i = 0
    while i < length - 1:
        min = i

        scan = i + 1
        while scan < length:
            if blocksList[scan].id < blocksList[min].id:
                min = scan
            scan += 1

        blocksList[i], blocksList[min] = blocksList[min], blocksList[i]
        i += 1

    # save the stuff
    fp = open (savefile, 'w')

    counter = 0
    for block in blocksList:
        fp.write (str (block.number))
        counter += 1
        if counter % 12 is 0:
            fp.write ('\n')

    fp.close()
    print 'Saved the level as: %s' % (savefile)
            
def load_level (level):
    map = []
    fp = open (level)
    lines = fp.readlines()

    # load map
    for line in lines:
        innermap = []
        for num in line:
            if num != '\n':
                innermap.append (num)
        map.append (innermap)

    fp.close()

    # construct level
    id = 0
    ypos = 0
    for y in range (12):
        xpos = 0
        for x in map[y]:
            Block (xpos, ypos, int (x), id).add (blocks)
            id += 1
            xpos += 40
            
        ypos += 20

def empty_level():
    id = 0
    ypos = 0
    for y in range (12):
        xpos = 0
        for x in range (12):
            Block (xpos, ypos, 0, id).add (blocks)
            id += 1
            xpos += 40
            
        ypos += 20

def load_files():
    background = pygame.Surface (screen.display.get_size()).convert()
    background.fill ((91, 159, 211))

    global gfx
    gfx = {1: load_image ('block1.jpeg'),
           2: load_image ('block2.jpeg'),
           3: load_image ('block3.jpeg'),
           4: load_image ('block4.jpeg'),
           5: load_image ('block5.jpeg'),
           6: load_image ('block6.jpeg'),
           7: load_image ('block7.jpeg'),
           'bg1': background}

def game_loop():
    clock = pygame.time.Clock()
    screen.display.blit (gfx['bg1'], (0,0))
    for block in blocks.sprites():
        if block.number != 0:
            block.draw()

    while 1:
        clock.tick (60)

        for event in pygame.event.get():
            if event.type is QUIT:
                return
            elif event.type is KEYDOWN:
                if event.key == K_ESCAPE:
                    return
                elif event.key == K_s:
                    save_level()
                elif event.key == K_c:
                    for block in blocks.sprites():
                        block.reset()

            elif event.type is MOUSEBUTTONDOWN:
                pos = pygame.mouse.get_pos()

                for block in blocks.sprites():
                    if block.rect.collidepoint (pos):
                        block.update()
                        break

        pygame.display.flip()

if __name__ == '__main__':
    main()



